<?php

use Elvez\AdminUtilV9 as AdminUtil;

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://elvez.co.jp
 * @since      1.0.0
 *
 * @package    Elvez_Manage_Contents
 * @subpackage Elvez_Manage_Contents/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Elvez_Manage_Contents
 * @subpackage Elvez_Manage_Contents/admin
 * @author     Elvez, Inc. <info@elvez.co.jp>
 */
class Elvez_Manage_Contents_Admin {
	/**
	 * 翻訳ファイルのドメイン
	 */
	const TEXT_DOMAIN = Elvez_Manage_Contents::TEXT_DOMAIN;

	/**
	 * トップレベルメニューのスラグ
	 */
	const ELVEZ_TOP_LEVEL_ADMIN_MENU = 'elvez_admin';

	/**
	 * 設定メニューの権限
	 */
	const CAPABILITY = 'manage_options';

	/**
	 * 設定ページのURLスラグ
	 */
	const MENU_SLUG = 'elvez-manage-contents';

	/**
	 * 設定メニューのアイコン
	 * https://developer.wordpress.org/resource/dashicons/
	 */
	const ICON_URL = 'dashicons-edit-large';

	/**
	 * 設定メニューの表示位置
	 */
	const POSITION = 99;

	/**
	 * オプションのグループ名
	 */
	const OPTION_GROUP = 'elvez-manage-contents';

	/**
	 * オプションフィールド名のプレフィックス
	 */
	const OPTION_PREFIX = 'elvez_manage_contents_';

	/**
	 * ショートコード
	 * @since	1.1.0	Woocommerce商品購入者のショートコード
	 * @since	1.1.4	ユーザエージェントを参照するショートコード
	 */
	const SHORTCODE_WC_PURCHASED = 'elv_mc_wc_purchased';
	const SHORTCODE_WC_NOT_PURCHASED = 'elv_mc_wc_not_purchased';
	const SHORTCODE_USER_AGENT = 'elv-mc-user-agent';
	const SHORTCODE_ROLE = 'elv-mc-role';

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;

		add_action('admin_init', [$this, 'register_settings']);
		add_action('admin_menu', [$this, 'set_menu_page']);

	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles( $hook_suffix ) {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Elvez_Manage_Contents_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Elvez_Manage_Contents_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		if ( $hook_suffix == 'elvez_page_elvez-manage-contents' ) {
			wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/elvez-manage-contents-admin.css', array(), $this->version, 'all' );
		}

	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Elvez_Manage_Contents_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Elvez_Manage_Contents_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		//wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/elvez-manage-contents-admin.js', array( 'jquery' ), $this->version, false );

	}

	/**
	 * Register setting fields.
	 *
	 * @since    1.0.0
	 */
	public function register_settings() {
		//add_option(self::OPTION_NAME, '1');
		//register_setting( self::OPTION_GROUP, self::OPTION_NAME);

		/**
		 * Subscriptions
		 */
		add_option(Elvez_Manage_Contents::OPTION_SUBSCRIBE_EMAIL, '');
		register_setting( self::OPTION_GROUP, Elvez_Manage_Contents::OPTION_SUBSCRIBE_EMAIL );
		add_option(Elvez_Manage_Contents::OPTION_SUBSCRIPTION_ID, '');
		register_setting( self::OPTION_GROUP, Elvez_Manage_Contents::OPTION_SUBSCRIPTION_ID );
	}


	/**
	 * Register top-level menu.
	 *
	 * @since    1.0.0
	 */
	public function register_top_level_menu() {
		global $menu;
		if ( !isset($menu[self::ELVEZ_TOP_LEVEL_ADMIN_MENU]) ) {
			// FIXME: Use icon image.
			// $icon = plugin_dir_url( __FILE__ ) . 'img/menu_icon.svg';

			add_menu_page(
				__( 'Elvez', self::TEXT_DOMAIN ),  // Page title
				__( 'Elvez', self::TEXT_DOMAIN ),  // Menu title
				self::CAPABILITY,
				self::ELVEZ_TOP_LEVEL_ADMIN_MENU,
				[$this, 'render_menu_page'], // Display first submenu page
				self::ICON_URL,
				self::POSITION
			);
		}
	}
	/**
	 * Set menu page.
	 *
	 * @since    1.0.0
	 */
	public function set_menu_page() {
		$this->register_top_level_menu();

		$this->admin_page_title = __( 'Manage Contents Admin', self::TEXT_DOMAIN );

		add_submenu_page(
			self::ELVEZ_TOP_LEVEL_ADMIN_MENU,
			$this->admin_page_title,  // Page title
			$this->admin_page_title,  // Menu title
			self::CAPABILITY,
			self::MENU_SLUG,
			[$this, 'render_menu_page'],  // Callback functions to render menu page
		);
		remove_submenu_page(self::ELVEZ_TOP_LEVEL_ADMIN_MENU, self::ELVEZ_TOP_LEVEL_ADMIN_MENU);
	}

	/**
	 * Re render menu page html.
	 *
	 * @since    1.0.0
	 */
	public function render_menu_page() {
		?>
		<div class="wrap">
			<?php AdminUtil::render_admin_page_title( $this->admin_page_title ); ?>

			<p><?php esc_html_e('Enclose the content you want to manage the display with below shortcodes.', self::TEXT_DOMAIN); ?></p>
			<?php $this->render_login_shortcode(); ?>

			<?php $this->render_wc_shortcode(); ?>

			<?php $this->render_role_shortcode(); ?>

			<?php $this->render_user_agent_shortcode(); ?>

			<form method="post" action="options.php">
				<?php settings_fields(self::OPTION_GROUP); ?>
				<?php do_settings_sections(self::OPTION_GROUP); ?>
				<!-- Subscription Settings -->
				<?php Elvez_Manage_Contents::render_subscription_form() ?>
				<?php submit_button(); ?>
			</form>
		</div>
		<?php
	}
	/**
	 * Render shortcodes to handle user login.
	 *
	 * @since    1.1.4
	 */
	public function render_login_shortcode() {
		?>
		<table class="form-table">
			<tr valign="top">
				<th scope="row"><?php esc_html_e( 'Show Contents only Login User', self::TEXT_DOMAIN ); ?></th>
				<td>
					<code>[elvez_login_contents]<?php esc_html_e( 'write contents here', self::TEXT_DOMAIN ); ?>[/elvez_login_contents]</code>
				</td>
			</tr>
			<tr valign="top">
				<th scope="row"><?php esc_html_e( 'Show Contents only Logout User', self::TEXT_DOMAIN ); ?></th>
				<td>
					<code class="elv-need-subscribe">[elvez_logout_contents]<?php esc_html_e( 'write contents here', self::TEXT_DOMAIN ); ?>[/elvez_logout_contents]</code>
				</td>
			</tr>
		</table>
		<?php
	}
	/**
	 * Render shortcodes to handle user login.
	 *
	 * @since    1.1.4
	 */
	public function render_wc_shortcode() {
		$sc_purchased = self::SHORTCODE_WC_PURCHASED;
		$sc_not_purchased = self::SHORTCODE_WC_NOT_PURCHASED;
		?>
		<table class="form-table">
			<tr valign="top">
				<th scope="row"><?php esc_html_e( 'Show Contents only purchased User with Product ID', self::TEXT_DOMAIN ); ?></th>
				<td>
					<code class="elv-need-subscribe"><?php echo "[$sc_purchased product_id=1]" ?><?php esc_html_e( 'write contents here', self::TEXT_DOMAIN ); ?><?php echo "[/$sc_purchased]" ?></code>
				</td>
			</tr>
			<tr valign="top">
				<th scope="row"><?php esc_html_e( 'Show Contents only NOT purchased User with Product ID', self::TEXT_DOMAIN ); ?></th>
				<td>
					<code class="elv-need-subscribe"><?php echo "[$sc_not_purchased product_id=1]" ?><?php esc_html_e( 'write contents here', self::TEXT_DOMAIN ); ?><?php echo "[/$sc_not_purchased]" ?></code>
				</td>
			</tr>
		</table>
		<?php
	}
	/**
	 * Render shortcodes to handle user agent.
	 *
	 * @since    1.1.4
	 */
	public function render_user_agent_shortcode() {
		$code = static::SHORTCODE_USER_AGENT;
		?>
		<table class="form-table">
			<tr valign="top">
				<th scope="row"><?php esc_html_e( 'Show contents if includes user agent', self::TEXT_DOMAIN ); ?></th>
				<td>
					<code><?php echo "[${code} includes=\"ie\"]" . __( 'write contents here', self::TEXT_DOMAIN ) . "[/${code}]"; ?></code>
				</td>
			</tr>
			<tr valign="top">
				<th scope="row"><?php esc_html_e( 'Show contents if excludes user agent', self::TEXT_DOMAIN ); ?></th>
				<td>
				<code><?php echo "[${code} excludes=\"chrome, firefox, edge\"]" . __( 'write contents here', self::TEXT_DOMAIN ) . "[/${code}]"; ?></code>
				</td>
			</tr>
			<tr valign="top">
				<th scope="row" style="padding-top: 0;"></th>
				<td style="padding-top: 0;">
					<p><?php esc_html_e('Support user agents are belows', self::TEXT_DOMAIN); ?></p>
					<?php
					$ua_list = static::get_ua_list();
					foreach( $ua_list as $key => $label ) {
						echo "<span style=\"margin-right: 1rem;\">${label}: <code>${key}</code></span>";
					}
					?>
				</td>
			</tr>
		</table>
		<?php
	}
	/**
	 * Return support user agent list
	 * TODO: support firefox
	 * @since	1.1.4
	 */
	static public function get_ua_list() {
		return array(
			'chrome' => 'Google Chrome',
			//'firefox' => 'Firefox',
			'safari' => 'Safari',
			'edge' => 'Microsoft Edge',
			'ie' => 'Internet Explorer',
		);
	}
	/**
	 * Render shortcodes to handle role.
	 *
	 * @since    1.1.5
	 */
	public function render_role_shortcode() {
		$code = self::SHORTCODE_ROLE;
		?>
		<table class="form-table">
			<tr valign="top">
				<th scope="row"><?php esc_html_e( 'Show Contents only user has role', self::TEXT_DOMAIN ); ?></th>
				<td>
					<code class=""><?php echo "[$code elv_role='role_id_1, role_id_2']" ?><?php esc_html_e( 'write contents here', self::TEXT_DOMAIN ); ?><?php echo "[/$code]" ?></code>
				</td>
			</tr>
			<tr valign="top">
				<th scope="row" style="padding-top: 0;"></th>
				<td style="padding-top: 0;">
					<p><?php esc_html_e('Support role ID defined by Elvez Role plugin.', self::TEXT_DOMAIN); ?></p>
					<p><?php esc_html_e('Specify roles separated by comma if allow multipe roles.', self::TEXT_DOMAIN); ?></p>
				</td>
			</tr>
		</table>
		<?php
	}
}
