<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       https://elvez.co.jp
 * @since      1.0.0
 *
 * @package    Elvez_Manage_Contents
 * @subpackage Elvez_Manage_Contents/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @package    Elvez_Manage_Contents
 * @subpackage Elvez_Manage_Contents/public
 * @author     Elvez, Inc. <info@elvez.co.jp>
 */
class Elvez_Manage_Contents_Public {
	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of the plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;

		add_action( 'init', [$this, 'register_shortcodes'] );
		add_action( 'init', [$this, 'register_dependencies'] );
		/**
		 * FIXME: Define in common class
		 * To put autoptimize files at bottom in head tag.
		 * Load autoptimized css files after getting CDN files.
		 */
		if ( !function_exists( 'elv_set_autoptimize_replacetag' ) ) {
			add_filter('autoptimize_filter_css_replacetag', 'elv_set_autoptimize_replacetag' );
			function elv_set_autoptimize_replacetag($replacetag) {
				return array("</head>","before");
			}
		}
	}

	/**
	 * Get registered version of script or style.
	 * 指定キーのスクリプトまたはスタイルが登録されていたらそのバージョン番号を返す
	 * 登録されていない場合は '0' を返す
	 *
	 * @since     1.0.1
	 * @param	$key	string
	 * @param	$type	string	'scripts' | 'styles'
	 * @return	$ver	string	Version string
	 */
	public static function get_registered_version( $key, $type='scripts') {

		$not_registerd_ver = '0';

		if ( $type == 'scripts' ) {
			$dependencies = wp_scripts();
		} else if ( $type == 'styles' ) {
			$dependencies = wp_styles();
		} else {
			return $not_registerd_ver;
		}

		if ( isset( $dependencies->registered[$key] ) ) {
			$registerd = $dependencies->registered[$key];
			return $registerd->ver;
		} else {
			return $not_registerd_ver;
		}

	}

	/**
	 * Register the stylesheets for the public-facing side of the site.
	 *
	 * @since    1.0.1
	 */
	public function register_dependencies() {

		$elvez_style_version = '1.2.2';
		$style_registered_ver = $this->get_registered_version( 'elvez-style', 'styles' );
		if ( version_compare( $elvez_style_version, $style_registered_ver, '>' ) ) {
			wp_styles()->remove( 'elvez-style' );
			wp_register_style( 'elvez-style', plugin_dir_url( dirname( __FILE__ ) ).'vendor/Elvez/css/elvez-style.css', array(), $elvez_style_version, 'all' );

		}

	}

	/**
	 * Register the stylesheets for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		wp_enqueue_style( 'bootstrap', 'https://stackpath.bootstrapcdn.com/bootstrap/4.5.0/css/bootstrap.min.css', array(), null, 'all' );

		wp_enqueue_style( 'elvez-style' );
		wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/elvez-manage-contents-public.css', array( 'elvez-style' ), $this->version, 'all' );

	}

	/**
	 * Register the JavaScript for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		wp_enqueue_script( 'popper.js', 'https://cdn.jsdelivr.net/npm/popper.js@1.16.0/dist/umd/popper.min.js', array( 'jquery' ), null, false );
		wp_enqueue_script( 'bootstrap', 'https://stackpath.bootstrapcdn.com/bootstrap/4.5.0/js/bootstrap.min.js', array( 'jquery', 'popper.js' ), null, false );

		wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/elvez-manage-contents-public.js', array( 'jquery' ), $this->version, false );

	}

	/**
	 * Register shotcodes.
	 *
	 * @since	1.0.0
	 * @since	1.0.3	Regster logout short code only has subscribed.
	 */
	public function register_shortcodes() {
		add_shortcode( 'elvez_login_contents', [$this, 'handle_elvez_login_contents'] );

		$is_subscribed = Elvez_Manage_Contents::is_subscribed();
		if ( $is_subscribed ) {
			add_shortcode( 'elvez_logout_contents', [$this, 'handle_elvez_logout_contents'] );
			add_shortcode( Elvez_Manage_Contents_Admin::SHORTCODE_WC_PURCHASED, [$this, 'handle_shortcode_wc_purchased'] );
			add_shortcode( Elvez_Manage_Contents_Admin::SHORTCODE_WC_NOT_PURCHASED, [$this, 'handle_shortcode_wc_not_purchased'] );
		}
		add_shortcode( Elvez_Manage_Contents_Admin::SHORTCODE_USER_AGENT, [$this, 'handle_shortcode_user_agent'] );
		add_shortcode( Elvez_Manage_Contents_Admin::SHORTCODE_ROLE, [$this, 'handle_shortcode_role'] );
	}

	/**
	 * Handle [elvez_login_contents] content.
	 *
	 * @since	1.0.0
	 * @params	array	$atts
	 * @params	string	$content
	 * @params	string	$tag
	 * @return	string	$html
	 */
	public function handle_elvez_login_contents( $atts, $content='', $tag ) {
		$is_show = is_user_logged_in();
		$html = $is_show ? do_shortcode( $content ) : '';  // do_shortcode: render nested shortcodes
		return $html;
	}

	/**
	 * Handle [elvez_logout_contents] content.
	 *
	 * @since	1.0.0
	 * @params	array	$atts
	 * @params	string	$content
	 * @params	string	$tag
	 * @return	string	$html
	 */
	public function handle_elvez_logout_contents( $atts, $content='', $tag ) {
		$is_show = !is_user_logged_in();
		$html = $is_show ? do_shortcode( $content ) : '';  // do_shortcode: render nested shortcodes
		return $html;
	}

	/**
	 * Handle shortcode content
	 *
	 * @since	1.1.0
	 * @param	array	$atts
	 * @param	string	$content
	 * @param	string	$tag
	 * @return	string	$html
	 */
	public function handle_shortcode_wc_purchased( $atts, $content='', $tag ) {
		// keyを変数名として値を代入
		extract( shortcode_atts( array(
			'product_id' => 0,
		), $atts ) );

		$user_id = get_current_user_id();
		$is_show = $this->is_purchased( $user_id, $product_id );

		$html = $is_show ? do_shortcode( $content ) : '';  // do_shortcode: render nested shortcodes
		return $html;
	}
	/**
	 * Handle shortcode content
	 *
	 * @since	1.1.0
	 * @param	array	$atts
	 * @param	string	$content
	 * @param	string	$tag
	 * @return	string	$html
	 */
	public function handle_shortcode_wc_not_purchased( $atts, $content='', $tag ) {
		// keyを変数名として値を代入
		extract( shortcode_atts( array(
			'product_id' => 0,
		), $atts ) );

		$user_id = get_current_user_id();
		$is_show = $this->is_purchased( $user_id, $product_id );

		$html = !$is_show ? do_shortcode( $content ) : '';  // do_shortcode: render nested shortcodes
		return $html;
	}
	/**
	 * Check user purchased product
	 *
	 * @since	1.1.0
	 * @param	int $user_id
	 * @param	int	$product_id
	 * @return	boolean
	 */
	public function is_purchased( $user_id, $product_id ) {
		if ( !function_exists( 'wc_customer_bought_product' ) ) {
			return false;
		} else {
			return wc_customer_bought_product( null, $user_id, $product_id );
		}
	}

	/**
	 * Handle shortcode content
	 *
	 * @since	1.1.4
	 * @param	array	$atts
	 * @param	string	$content
	 * @param	string	$tag
	 * @return	string	$html
	 */
	public function handle_shortcode_user_agent( $atts, $content='', $tag ) {
		// keyを変数名として値を代入
		extract( shortcode_atts( array(
			'includes' => '',
			'excludes' => '',
		), $atts ) );

		$is_show = true;

		if( $includes ) {
			$is_show = false;
			$ary = $this->parse_ua_param( $includes );
			foreach( $ary as $key ) {
				if( $this->is_ua( $key ) ) {
					$is_show = true;
					break;
				}
			}
 		} else if( $excludes ) {
			$is_show = true;
			$ary = $this->parse_ua_param( $excludes );
			foreach( $ary as $key ) {
				if( $this->is_ua( $key ) ) {
					$is_show = false;
					break;
				}
			}
 		}
		return $is_show ? do_shortcode( $content ) : '';

	}
	/**
	 * Parse shortcode param within user agent list
	 * @since	1.1.4
	 * @param	string	$atr_keys
	 * @return	array
	 */
	function parse_ua_param( $str_keys ) {
		$ua_list = Elvez_Manage_Contents_Admin::get_ua_list();
		$keys = array_map( 'trim', explode(',', $str_keys) );
		$ary = [];
		foreach( $keys as $key ) {
			if( array_key_exists($key, $ua_list) ) {
				$ary[] = $key;
			}
		}
		return $ary;
	}
	/**
	 * Check request user agent is some browser
	 * @since	1.1.4
	 * @param	string	$key
	 * @return	boolean
	 */
	function is_ua( $key ) {
		switch( $key ) {
			case 'chrome':
				global $is_chrome;
				return $is_chrome;
			//case 'firefox':
			//	TODO;
			case 'safari':
				global $is_safari;
				return $is_safari;
			case 'edge':
				global $is_edge;
				return $is_edge;
			case 'IE':
			case 'ie':
				global $is_IE;
				return $is_IE;
			default:
				return false;
		}
	}

	/**
	 * Handle [elv-mc-role] content.
	 *
	 * @since	1.1.5
	 * @param	array	$atts
	 * @param	string	$content
	 * @param	string	$tag
	 * @return	string
	 */
	public function handle_shortcode_role( $atts, $content='', $tag ) {
		extract( shortcode_atts( array(
			'class' => '',
			'elv_role' => null,
			'role' => null,
		), $atts ) );

		$user_id = get_current_user_id();
		if( !$user_id ) {
			return '';
		}

		if( $elv_role && class_exists( 'Elvez_Role' ) ) {
			$roles = array_map( 'trim', explode(',', $elv_role) );
			$user_role = Elvez_Role::get_user_role_id( $user_id );
			if( in_array( $user_role, $roles ) ) {
				return do_shortcode( $content );
			}
		}

		return '';
	}

}
