<?php
namespace Elvez;

class AdminUtilV9 {
	/**
	 * 翻訳テキストドメイン
	 *
	 * @since    1
	 */
	const TEXT_DOMAIN = 'elvez-admin-util';

	/**
	 * The singletone instance
	 *
	 * @since    1
	 */
	protected static $_instance = null;

	/**
	 * The datetime format
	 *
	 * @since    1
	 */
	protected static $_format = 'Y-m-d H:i:s';

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1
	 */
	public function __construct() {
		/**
		 * 翻訳ドメイン登録
		 */
		add_action( 'plugins_loaded', [$this, 'load_textdomain'] );
		/**
		 * スクリプト登録
		 */
		add_action( 'init', [$this, 'register_dependencies'] );
		add_action( 'admin_enqueue_scripts', [$this, 'enqueue_scripts'] );

		/**
		 * WP管理アドレスへのメール送信
		 */
		add_action( 'elvez_admin_util_send_admin_email', [$this, 'send_admin_email'], 10, 3 );

	}
    /**
     * Return singleton instance.
     *
     * @since   3
     * @return  self
     */
    public static function get_instance() {
        if ( is_null( self::$_instance ) ) {
            self::$_instance = new self();
		}
        return self::$_instance;
	}
	/**
	 * Load language files.
	 *
	 * @since	3
	 */
	function load_textdomain() {
		load_plugin_textdomain(
			self::TEXT_DOMAIN,
			false,
			dirname( plugin_basename( __FILE__ ) ) . '/languages/'
		);
	}
	/**
	 * Get registered version of script or style.
	 * 指定キーのスクリプトまたはスタイルが登録されていたらそのバージョン番号を返す
	 * 登録されていない場合は '0' を返す
	 *
	 * @since	4
	 * @param	$key	string
	 * @param	$type	string	'scripts' | 'styles'
	 * @return	$ver	string	Version string
	 */
	public static function get_registered_version( $key, $type='scripts') {
		$not_registerd_ver = '0';
		if ( $type == 'scripts' ) {
			$dependencies = wp_scripts();
		} else if ( $type == 'styles' ) {
			$dependencies = wp_styles();
		} else {
			return $not_registerd_ver;
		}
		if ( isset( $dependencies->registered[$key] ) ) {
			$registerd = $dependencies->registered[$key];
			return $registerd->ver;
		} else {
			return $not_registerd_ver;
		}
	}
	/**
	 * Register script considered version.
	 *
	 * @since	4
	 */
	public function register_dependencies() {
		$tag = 'elvez-admin-util';
		$version = '5';
		$registered_ver = self::get_registered_version( $tag, 'scripts' );
		if ( version_compare( $version, $registered_ver, '>' ) ) {
			wp_scripts()->remove( $tag );
			wp_register_script( $tag, plugin_dir_url( __FILE__ ) . 'js/elvez-admin-util.js', array( 'jquery' ), $version, false );
		}
	}
	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    4
	 */
	public function enqueue_scripts() {
		wp_enqueue_media(); // For Image selector
		wp_enqueue_script( 'elvez-admin-util' );
	}
	/**
	 * Register option to option group
	 * @since	2
	 * @param	string	$group
	 * @param	string	$name
	 * @param	mixed	$default
	 */
	public static function register_option_setting( $group, $name, $default ) {
		add_option($name, $default);
		register_setting( $group, $name);
	}

	/**
	 * Get name string with nested key
	 * @since	3
	 * @param	string	$opt_name
	 * @param	array	$nested_keys
	 * @return	string	$name
	 */
	public static function get_name( $name, $nested_keys=array()) {
		foreach( $nested_keys as $key ) {
			$name = $name . "[$key]";
		}
		return $name;
	}
	/**
	 * Get value with nested key
	 * @since	3
	 * @param	mixed	$value
	 * @param	array	$nested_keys
	 * @return	mixed	$value
	 */
	public static function get_value( $value, $nested_keys=array()) {
		if ( $nested_keys ) {
			$value = self::get_nested_array_value( $value, $nested_keys );
		}
		return $value;
	}

	/**
	 * Get array value by nested key
	 * @since	2
	 * @param	array	$haystack
	 * @param	array	$nested_keys
	 * @return	mixed	$value	A value of nested key if exists.
	 */
	public static function get_nested_array_value( $haystack, $nested_keys) {
		$not_found = false;
		$stack = $haystack;
		foreach ( $nested_keys as $key ) {
			if ( is_array( $stack ) && isset( $stack[$key] ) ) {
				$stack = $stack[$key];
			} else {
				$not_found = true;
				break;
			}
		}
		if ( $not_found ) {
			$value = null;
		} else {
			$value = $stack;
		}
		return $value;
	}
	/**
	 * Get value from option or user meta or post meta
	 * @since	8
	 * @param	string	$opt_name
	 * @param	array	$args	Array of options for input field
	 * 		string	meta_type	'post' or 'user'
	 * 		int		meta_id		An ID of meta type object
	 * @return	mixed
	 */
	public static function get_opt_value( $opt_name, $args=array() ) {
		$meta_type = $args['meta_type'] ?? null;
		if ( $meta_type === 'user' ) {
			$meta_id = $args['meta_id'] ?? null;
			$opt_value = get_user_meta( $meta_id, $opt_name, true );
		} else if ( $meta_type === 'post' ) {
			$meta_id = $args['meta_id'] ?? null;
			$opt_value = get_post_meta( $meta_id, $opt_name, true );
		} else {
			$opt_value = get_option( $opt_name, '' );
		}
		return $opt_value;
	}

	/**
	 * Render admin page title
	 *
	 * @since	8
	 */
	public static function render_admin_page_title( $page_title ) {
		?>
		<h1 class="elv-admin-page-title"><?php echo sprintf( __( '%s Settings', self::TEXT_DOMAIN ), $page_title ); ?></h1>
		<?php
	}

	/**
	 * Render input field typed text in form
	 * @since	1
	 * @since	5	Support nested keys
	 * @since	8	Support Post and User meta
	 * @param	string	$opt_name
	 * @param	int		$size	Number of input field width.
	 * @param	array	$nested_keys	Array of keys to get value in option array
	 * @param	array	$args	Array of options for input field
	 * 		bool	required
	 * 		string	description
	 * 		string	meta_type	'post' or 'user'
	 * 		int		meta_id		An ID of meta type object
	 */
	public static function render_input_text_option( $opt_name, $size=60, $nested_keys=array(), $args=array() ) {
		$name = self::get_name( $opt_name, $nested_keys );
		$opt_value = self::get_opt_value( $opt_name, $args );
		$value = self::get_value( $opt_value, $nested_keys );
		if ( $value == null && isset( $args['default'] ) ) {
			$value = $args['default'];
		}
		$required = isset( $args['required'] ) && $args['required'] ? 'required' : '';

		?>
		<input type="text"
			size="<?php echo $size ?>"
			name="<?php echo esc_attr( $name ); ?>"
			value="<?php echo esc_attr( $value ); ?>"
			<?php echo $required ?>
		/>
		<?php
		if ( isset( $args['description'] ) && $args['description'] ) {
			?>
			<p class="description"><?php echo esc_html( $args['description'] ); ?></p>
			<?php
		}
	}
	/**
	 * Render input field typed text in form
	 * @since	1
	 * @since	2	Apply nested option
	 * @since	8	Support Post and User meta
	 * @param	string	$name
	 * @param	array	$nested_keys
	 * @param	array	$attrs
	 * @param	bool	$show_reset
	 * @param	array	$args	Uploader args
	 * 		string	meta_type	'post' or 'user'
	 * 		int		meta_id		An ID of meta type object
	 */
	public static function render_input_color_option( $opt_name, $nested_keys=array(), $attrs=array(), $show_reset=true, $args=array() ) {
		$name = self::get_name( $opt_name, $nested_keys );
		$opt_value = self::get_opt_value( $opt_name, $args );
		$value = self::get_value( $opt_value, $nested_keys );
		$attr_strings = http_build_query($attrs, '', ' ');

?>
		<input type="color"
			name="<?php echo esc_attr( $name ); ?>"
			value="<?php echo esc_attr( $value ); ?>"
			<?php echo esc_attr( $attr_strings ); ?> />
		<?php
		/* TODO: Impliment for reset color
		 * valueにから文字は設定できないためチェックボックス等で対応してみる
		if ( $show_reset ) {
			$reset_btn_class = 'button elv-reset-form-' . $name;
			?>
			<button class="<?php echo $reset_btn_class; ?>"><?php esc_html_e( 'Reset', self::TEXT_DOMAIN ); ?></button>
			<script>
				(function( $ ) {
					const $input = $("input[name='<?php echo esc_attr( $name ); ?>']");
					$(".<?php echo $reset_btn_class; ?>").click( function(e) { $input.val(''); } );
				})( jQuery );
			</script>
			<?php
		}
		*/
	}
	/**
	 * Render image input.
	 *
	 * @since	3
	 * @since	8	Support Post and User meta
	 * @param	string	$name
	 * @param	array	$nested_keys	Array of keys to get value in option array
	 * @param	array	$attrs	Url attributes
	 * @param	array	$args	Uploader args
	 * 		string	meta_type	'post' or 'user'
	 * 		int		meta_id		An ID of meta type object
	 */
	public static function render_input_image_option( $opt_name, $nested_keys=array(), $attrs=array(), $args=array() ) {
		$name = self::get_name( $opt_name, $nested_keys );
		$opt_value = self::get_opt_value( $opt_name, $args );
		$value = self::get_value( $opt_value, $nested_keys );
		$attr_strings = http_build_query($attrs, '', ' ');

		$src = $value ? wp_get_attachment_url( $value ) : '';
		$uploader_title = isset( $args['uploader_title'] ) ? $args['uploader_title'] : __( 'Select Image', self::TEXT_DOMAIN );
		$uploader_button_text = isset( $args['uploader_button_text'] ) ? $args['uploader_button_text'] : __( 'Submit', self::TEXT_DOMAIN );

		?>
		<script>
			const ELV_ADMINUTIL_IMAGE_OPTION = {
				uploader: {
					title: '<?php echo esc_html( $uploader_title ); ?>',
					button_text: '<?php echo esc_html( $uploader_button_text ); ?>',
				}
			}
		</script>

		<div class="elv-adminutil-image-option">
			<div class='elv-adminutil-image-option-preview-wrapper'>
				<img class='elv-adminutil-image-option-preview'
					src='<?php echo esc_attr( $src ); ?>'
					height='100'>
			</div>
			<input type="hidden"
				class="elv-adminutil-image-option-value"
				name="<?php echo esc_attr( $name ); ?>"
				value=<?php echo esc_attr( $value ); ?> />
			<button type="button"
				class="elv-adminutil-image-option-select button"><?php esc_html_e( 'Select Image' , self::TEXT_DOMAIN ); ?></button>
			<button type="button"
				class="elv-adminutil-image-option-remove button"
				style="display: <?php echo $src ? '': 'none' ?>"
				><?php esc_html_e( 'Remove' , self::TEXT_DOMAIN ); ?></button>
		<div>
		<?php
	}

	/**
	 * Send email for WP admin
	 *
	 * @since	6
	 * @param	string	$subject
	 * @param	string	$message
	 * @param	array	$args
	 * @return	boolean
	 */
	public static function send_admin_email( $subject, $message, $args=array() ) {
		$to = get_option( 'admin_email' );
		if ( $to ) {
			$headers = isset( $args['headers'] ) ? $args['headers'] : '';
			$attachments = isset( $args['attachments'] ) ? $args['attachments'] : array();
			return wp_mail( $to, $subject, $message, $headers, $attachments );
		} else {
			return false;
		}
	}
	/**
	 * Return default mail subject
	 *
	 * @since	6
	 * @param	string	$subject
	 * @return	string
	 */
	public static function get_admin_mail_subject( $subject='' ) {
		$site_name = get_bloginfo('name');
		return "[$site_name]$subject";
	}





}
