<?php
namespace Elvez;

use WP_Error;


class DBUtilV4 {
	/**
	 * 翻訳テキストドメイン
	 *
	 * @since    1
	 */
	const TEXT_DOMAIN = 'elvez_db_util';
	/**
	 * DBスキーマバージョン
	 * @since 2
	 */
	const OPTION_SCHEMA_VERSION = 'elvez_db_util_shcema_version';
	/**
	 * The singletone instance
	 *
	 * @since    1
	 */
	protected static $_instance = null;
	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1
	 */
	public function __construct() {
		/**
		 * 翻訳ドメイン登録
		 */
		add_action( 'plugins_loaded', [$this, 'load_textdomain'] );

	}
    /**
     * Return singleton instance.
     *
     * @since   1
     * @return  DBUtil
     */
    public static function get_instance() {
        if ( is_null( self::$_instance ) ) {
            self::$_instance = new self();
        }
        return self::$_instance;
    }
	/**
	 * Load language files.
	 *
	 * @since	1
	 */
	function load_textdomain() {
		load_plugin_textdomain(
			self::TEXT_DOMAIN,
			false,
			dirname( plugin_basename( __FILE__ ) ) . '/languages/'
		);
	}

	/**
	 * Return table name
	 * @since	1
	 * @return string $table
	 */
	static function get_table_name() {
		global $wpdb;
		$table = $wpdb->prefix . 'elv_historical_user_meta';
		return $table;
	}

	/**
	 * Create or update database for message history
	 *
	 * @since	1
	 */
	static public function create_or_update_db() {
		global $wpdb;

		if ( !self::compare_schema_version() ) {
			return;
		}

		$table = self::get_table_name();
		$charset_collate = $wpdb->get_charset_collate();
		$max_index_length = 191;

		$sql = "CREATE TABLE $table (
			id bigint(20) unsigned NOT NULL AUTO_INCREMENT,
			user_id bigint(20) unsigned NOT NULL default '0',
			meta_key varchar(255) default NULL,
			meta_value longtext,
			datetime datetime NOT NULL default '0000-00-00 00:00:00',
			PRIMARY KEY (id),
			KEY user_id (user_id),
			KEY meta_key (meta_key($max_index_length))
		) $charset_collate;";

		require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
		dbDelta( $sql );
	}
	/**
	 * Compare and update schema version
	 * @since 2
	 * @return bool
	 */
	static function compare_schema_version(){
		$schema_version = '1';
		add_option( self::OPTION_SCHEMA_VERSION, $schema_version );
		$current = get_option( self::OPTION_SCHEMA_VERSION );
		if ( !$current || version_compare( $schema_version, $current ) >= 0 ) {
			update_option( self::OPTION_SCHEMA_VERSION, $schema_version );
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Record historical user meta data
	 *
	 * @since	1
	 * @param int		$user_id	ID of the object metadata is for.
	 * @param string	$meta_key	Metadata key.
	 * @param mixed		$meta_value	Metadata value. Serialized if non-scalar.
	 * @return bool
	 */
	static public function record_historical_user_meta( $user_id, $meta_key, $meta_value ) {
		global $wpdb;

		if ( ! $meta_key || ! is_numeric( $user_id ) ) {
			return false;
		}
		$user_id = absint( $user_id );
		if ( ! $user_id ) {
			return false;
		}

		$raw_meta_key = $meta_key;
		$meta_key     = wp_unslash( $meta_key );
		$meta_value   = wp_unslash( $meta_value );
		$meta_value   = maybe_serialize( $meta_value );
		$datetime     = current_time( 'mysql', true );

		$data  = compact(
			array(
				'user_id',
				'meta_key',
				'meta_value',
				'datetime',
			)
		);

		$table = self::get_table_name();
		$result = $wpdb->insert( $table, $data );
		if ( ! $result ) {
			return false;
		}
		/**
		 * @since 1
		 *
		 * @param int    $id			ID of updated metadata entry.
		 * @param int    $user_id		ID of the object metadata is for.
		 * @param string $meta_key		Metadata key.
		 * @param mixed  $meta_value	Metadata value. Serialized if non-scalar.
		 * @param string $datetime		Datetime string.
		 */
		do_action( 'elv_db_util_record_historical_user_meta', $user_id, $meta_key, $meta_value, $datetime );

		return true;
	}
	/**
	 * Get records from historical user meta table
	 *
	 * @since	1
	 * @since	4	Allow specify record limit
	 * @param int    $user_id	ID of the object metadata is for.
	 * @param string $meta_key	Metadata key.
	 * @param string $where		A string of filtering SQL
	 * @param int	 $limit		A number of records limit
	 * @return array $records
	 */
	static public function get_historical_user_meta( $user_id, $meta_key, $where='', $limit=0 ) {
		global $wpdb;
		$table = self::get_table_name();
		$sort_key = 'datetime';

		$sql = "SELECT * FROM $table WHERE user_id=$user_id AND meta_key='$meta_key'";
		if ( $where ) {
			$sql .= " AND $where";
		}
		$sql .= " ORDER BY $sort_key DESC";
		if ( $limit ) {
			$sql .= " LIMIT $limit";
		}

		$records = $wpdb->get_results( $sql );
		return $records;
	}

	/**
	 * Get records from historical user meta table for all users
	 *
	 * @since	3
	 * @param string $meta_key	Metadata key.
	 * @param string $where		A string of filtering SQL
	 * @param int $limit	A number of max records
	 * @return array $records
	 */
	static public function get_historical_user_meta_all( $meta_key=null, $where='', $limit=null ) {
		global $wpdb;
		$table = self::get_table_name();
		$sort_key = 'datetime';

		$sql = "SELECT * FROM $table";
		if ( $meta_key && $where ) {
			$sql .= " WHERE meta_key='$meta_key' AND $where";
		} else if ( $meta_key ) {
			$sql .= " WHERE meta_key='$meta_key'";
		} else if ( $where ) {
			$sql .= " WHERE $where";
		}
		$sql .= " ORDER BY $sort_key DESC";
		if ( $limit ) {
			$sql .= " LIMIT $limit";
		}

		$records = $wpdb->get_results( $sql );
		return $records;
	}

	/**
	 * Rreturn daily records
	 * @since 1
	 * @param int $user_id
	 * @param string $key
	 * @param string $date	A date string considered timezone offset
	 * @return array $records
	 */
	static public function get_daily_records( $user_id, $key, $date ) {
		$offest = (float)get_option('gmt_offset');
		$start_date = date( 'Y-m-d H:i:s', strtotime( "$date -$offest hour" ) );
		$end_date = date( 'Y-m-d H:i:s', strtotime( "$date +1day -$offest hour" ) );
		$where = "datetime >= '$start_date' AND datetime < '$end_date'";
		$records = self::get_historical_user_meta( $user_id, $key, $where );
		return $records;
	}

	/**
	 * Update record value
	 * @since 4
	 * @param int $record_id
	 * @return array $meta_value
	 */
	static public function update_record_value( $record_id, $meta_value ) {
		global $wpdb;
		$table = self::get_table_name();

		$meta_value   = wp_unslash( $meta_value );
		$meta_value   = maybe_serialize( $meta_value );

		$args = array(
			'meta_value' => $meta_value,
		);
		$where = array(
			'id' => $record_id,
		);
		return $wpdb->update($table, $args, $where);
	}

}