<?php
namespace Elvez;

use WP_Error;


class DynamicLinkUtilV1 {
	/**
	 * 翻訳テキストドメイン
	 *
	 * @since    1
	 */
	const TEXT_DOMAIN = 'elvez_dynamic_link_util';

	/**
	 * FirebaseのAPIキー
	 *
	 * @since    1
	 */
	const OPTION_FIREBASE_API_KEY = 'elz_dlu_firebase_api_key';
	/**
	 * ダイナミックリンクのドメイン
	 *
	 * @since    1
	 */
	const OPTION_DYNAMIC_LINK_DOMAIN = 'elz_dlu_dynamic_link_domain';
	/**
	 * ダイナミックリンク作成のエンドポイント
	 *
	 * @since    1
	 */
	const DYNAMIC_LINKS_ENDPOINT = 'https://firebasedynamiclinks.googleapis.com/v1/shortLinks';


	/**
	 * The singletone instance
	 *
	 * @since    1.
	 */
	protected static $_instance = null;

	/**
	 * The datetime format
	 *
	 * @since    1
	 */
	protected static $_format = 'Y-m-d H:i:s';

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1
	 */
	public function __construct() {
		/**
		 * 翻訳ドメイン登録
		 */
		add_action( 'plugins_loaded', [$this, 'load_textdomain'] );
	}
    /**
     * Return singleton instance.
     *
     * @since   1
     * @return  DynamicLinkUtil
     */
    public static function get_instance() {
        if ( is_null( self::$_instance ) ) {
            self::$_instance = new self();
        }
        return self::$_instance;
    }
	/**
	 * Load language files.
	 *
	 * @since	1
	 */
	function load_textdomain() {
		load_plugin_textdomain(
			self::TEXT_DOMAIN,
			false,
			dirname( plugin_basename( __FILE__ ) ) . '/languages/'
		);
	}

	/**
	 * Create Dynamic link
	 *
	 * @since	1
	 * @param	string	$link	A string the url to convert
	 * @return	string	$short_link	A string of shorted url
	 */
	public static function get_short_link( $link ) {
		$domain = get_option( self::OPTION_DYNAMIC_LINK_DOMAIN, null );
		$api_key = get_option( self::OPTION_FIREBASE_API_KEY, null );

		if ( !$domain || !$api_key ) {
			return $link;
		}

		$long_link = urlencode( $link );
		$recbody = '{
			"longDynamicLink": "https://'.$domain.'/?link='.$long_link.'",
			"suffix": {
				"option": "SHORT"
			}
		}';

		$endpont = self::DYNAMIC_LINKS_ENDPOINT;
		$url = "$endpont?key=$api_key";

		$ch = curl_init();
		curl_setopt_array($ch, array(
			CURLOPT_URL => $url,
			CURLOPT_HTTPHEADER => ['Content-Type: application/json'],
			CURLOPT_RETURNTRANSFER => true,
			CURLOPT_POSTFIELDS => $recbody,
			CURLOPT_CONNECTTIMEOUT => 5,
		));

		$res = curl_exec($ch);
		$result = json_decode($res, true);
		curl_close($ch);

		if ( isset( $result['shortLink'] ) ) {
			return $result['shortLink'] ;
		} else {
			return $link;
		}
	}

	/**
	 * Register options
	 *
	 * @since	1
	 * @param	string	$option_group	A string of option group to register setting
	 */
	public static function register_options( $option_group) {
		add_option(self::OPTION_FIREBASE_API_KEY, '');
		register_setting( $option_group, self::OPTION_FIREBASE_API_KEY);

		add_option(self::OPTION_DYNAMIC_LINK_DOMAIN, '');
		register_setting( $option_group, self::OPTION_DYNAMIC_LINK_DOMAIN);
	}
	/**
	 * Render option form to firebase API
	 *
	 * @since	1
	 */
	public static function render_option_settings() {
		$api = self::OPTION_FIREBASE_API_KEY;
		$domain = self::OPTION_DYNAMIC_LINK_DOMAIN;
		?>
		<h3 class=""><?php esc_html_e( 'Dynamic Link' , self::TEXT_DOMAIN ); ?></h3>
		<p class="description"><?php esc_html_e( 'This setting may be overridden by other plugins.', self::TEXT_DOMAIN ); ?></p>
		<table class="form-table">
			<tr valign="top">
				<th scope="row"><?php esc_html_e( 'Firebase Web API Key', self::TEXT_DOMAIN ); ?></th>
				<td>
					<input type="password"
						name="<?php echo $api; ?>"
						value="<?php echo esc_attr( get_option($api) ); ?>" />
				</td>
			</tr>
			<tr valign="top">
				<th scope="row"><?php esc_html_e( 'Dynamic Link Domain', self::TEXT_DOMAIN ); ?></th>
				<td>
					<input type="text"
						name="<?php echo $domain; ?>"
						value="<?php echo esc_attr( get_option($domain) ); ?>" />
				</td>
			</tr>
		</table>
		<?php
	}

}