<?php
namespace Elvez;

use Elvez\AdminUtilV9 as AdminUtil;

use Aws\Sns\SnsClient as SnsClient;
use WP_Error;


class PhoneNumberUtilV6 {
	/**
	 * 翻訳テキストドメイン
	 *
	 * @since    1.0.0
	 */
	const TEXT_DOMAIN = 'elvez_phone_number_util';

	/**
	 * 携帯番号のユーザメタフィールド
	 *
	 * @since    1.0.0
	 */
	const USER_META_PHONE_NUMBER = 'elv_phone_number';

	/**
	 * SMS認証情報のオプション
	 *
	 * @since    6
	 */
	const OPTION_SMS_CREDENTIALS = 'elv_phone_number_util_sms_credentials';
	/**
	 * SMS SnderIDのオプション
	 *
	 * @since    6
	 */
	const OPTION_SMS_SENDER_ID = 'elv_phone_number_util_sms_snder_id';

	/**
	 * The singletone instance
	 *
	 * @since    1.0.0
	 */
	protected static $_instance = null;

	/**
	 * The datetime format
	 *
	 * @since    1.0.0
	 */
	protected static $_format = 'Y-m-d H:i:s';

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {
		/**
		 * 翻訳ドメイン登録
		 */
		add_action( 'plugins_loaded', [$this, 'load_textdomain'] );
		/**
		 * ユーザ管理画面に携帯番号を追加
		 */
		add_action('pre_user_query', [$this, 'add_search_by_phone_number'] );
		add_action( 'show_user_profile', [$this, 'add_phone_number_field'] );
		add_action( 'edit_user_profile', [$this, 'add_phone_number_field'] );
		add_action( 'personal_options_update', [$this, 'save_phone_number_field'] );
		add_action( 'edit_user_profile_update', [$this, 'save_phone_number_field'] );
		add_filter( 'manage_users_columns', [$this, 'add_columns'] );
		add_action( 'manage_users_custom_column', [$this, 'render_colmuns'], 10, 3 );
	}

    /**
     * Return singleton instance.
     *
     * @since   1.0.1
     * @return  PhoneNumberUtil
     */
    public static function get_instance() {
        if ( is_null( self::$_instance ) ) {
            self::$_instance = new self();
        }
        return self::$_instance;
    }
	/**
	 * Load language files.
	 *
	 * @since	5
	 */
	function load_textdomain() {
		load_plugin_textdomain(
			self::TEXT_DOMAIN,
			false,
			dirname( plugin_basename( __FILE__ ) ) . '/languages/'
		);
	}

	/**
	 * Show phone number field in profile page.
	 * @param $user	WP_User
	 * @since 1.0.0
	 */
	function add_phone_number_field( $user ) {
		?>
		<h2><?php esc_html_e( 'Elvez Plugins', self::TEXT_DOMAIN ); ?></h2>
		<table class="form-table">
			<tr>
				<th scope="row"><?php esc_html_e( 'Mobile Phone Number', self::TEXT_DOMAIN ); ?></th>
				<td>
					<input type="tel"
						name="<?php echo esc_attr( self::USER_META_PHONE_NUMBER ); ?>"
						value="<?php echo esc_attr( get_user_meta( $user->ID, self::USER_META_PHONE_NUMBER, true ) ); ?>" />
					<p class="description"><?php esc_html_e( 'Should be able to receive SMS.' , self::TEXT_DOMAIN); ?></p>
				</td>
			</tr>
		</table>
		<?php
	}
    /**
     * Set phone number in number
     * @since 1.0.0
     * @param	$user_id	int
     */
    function save_phone_number_field( $user_id ) {
        if (! current_user_can('edit_user', $user_id))
            return false;

        $value = sanitize_key( $_POST[self::USER_META_PHONE_NUMBER] );

        // FIXME: should be validate to unique
        update_user_meta( $user_id, self::USER_META_PHONE_NUMBER, $value);
    }

	/**
	 *  Add columns to user list.
	 *
	 * @since	5
	 * @var		array	$columns	admin user list colmuns
	 * @return	array
	 */
	function add_columns( $columns ) {
        $columns[self::USER_META_PHONE_NUMBER] = __( 'Phone Number', self::TEXT_DOMAIN );
		return $columns;
	}
	/**
	 * Render custom colmun in user list.
	 *
	 * @since	5
     * @param   $value          string  custom column output
     * @param   $column_name    string
     * @param   $user_id        int
	 */
	public function render_colmuns( $value, $column_name, $user_id ) {
		switch ( $column_name ) {
			case self::USER_META_PHONE_NUMBER:
				$value = self::get_phone_number( $user_id );
				break;
		}
        return $value;
	}

	/**
	 * Filter user in user search
	 * @since	5
	 * @param	$wp_user_query
	 * @return	$wp_user_query
	 */
	function add_search_by_phone_number($wp_user_query) {
		global $pagenow;

		if ( is_admin() && 'users.php' == $pagenow ) {
			if ( false === strpos( $wp_user_query->query_where, '@' ) && !empty( $_GET['s'] ) ) {
				global $wpdb;

				$user_ids = array();
				$user_ids_per_term = array();

				// Usermeta fields to search
				$usermeta_keys = array( self::USER_META_PHONE_NUMBER );
				$query_string_meta = '';
				$search_terms = sanitize_text_field( $_GET['s'] );
				$search_terms_array = explode( ' ', $search_terms );

				// Search users for each search term (word) individually
				foreach ($search_terms_array as $search_term) {
					// reset ids per loop
					$user_ids_per_term = array();

					// add all custom fields into the query
					if (!empty($usermeta_keys)) {
						$query_string_meta = "meta_key='" . implode("' OR meta_key='", esc_sql($usermeta_keys)) . "'";
					}
					// Query usermeta table
					$usermeta_results = $wpdb->get_results( $wpdb->prepare( "SELECT DISTINCT user_id FROM $wpdb->usermeta WHERE (" . $query_string_meta . ") AND LOWER(meta_value) LIKE '%%%s%%'", $search_term ) );
					foreach ($usermeta_results as $usermeta_result) {
						if (!in_array($usermeta_result->user_id, $user_ids_per_term)) {
							array_push($user_ids_per_term, $usermeta_result->user_id);
						}
					}
					// Query users table
					$users_results = $wpdb->get_results($wpdb->prepare("SELECT DISTINCT ID FROM $wpdb->users WHERE LOWER(user_nicename) LIKE '%%%s%%' OR LOWER(user_email) LIKE '%%%s%%' OR LOWER(display_name) LIKE '%%%s%%'", $search_term, $search_term, $search_term));
					foreach ($users_results as $users_result) {
						if (!in_array($users_result->ID, $user_ids_per_term)) {
							array_push($user_ids_per_term, $users_result->ID);
						}
					}
					// Limit results to matches of all search terms
					if (empty($user_ids)) {
						$user_ids = array_merge($user_ids, $user_ids_per_term);
					} else {
						if (!empty($user_ids_per_term)) {
							$user_ids = array_unique(array_intersect($user_ids, $user_ids_per_term));
						}
					}
				}

				// Convert IDs to comma separated string
				$ids_string = implode(',', $user_ids);
				if (!empty($ids_string)) {
					// network users search (multisite)
					$wp_user_query->query_where = str_replace("user_nicename LIKE '" . $search_terms . "'", "ID IN(" . $ids_string . ")", $wp_user_query->query_where);
					// site (blog) users search
					$wp_user_query->query_where = str_replace("user_nicename LIKE '%" . $search_terms . "%'", "ID IN(" . $ids_string . ")", $wp_user_query->query_where);
					// network/site users search by number (WordPress assumes user ID number)
					$wp_user_query->query_where = str_replace("ID = '" . $search_terms . "'", "ID = '" . $search_terms . "' OR ID IN(" . $ids_string . ")", $wp_user_query->query_where);
				}
			}
		}

		return $wp_user_query;
	}


	/**
	 * Get user by phone number
	 *
	 * @since	1
	 * @since	4	Modify to function is static
	 * @param	$phone_number	string
	 * @return	WP_USER
	 */
	static public function get_user_by_phone_number( $phone_number ) {
		if ( !$phone_number ) {
			return null;
		}

		$meta_key = self::USER_META_PHONE_NUMBER;

		$args = array(
			'meta_key' => $meta_key,
			'meta_value' => $phone_number
		);
		$users = get_users( $args );

		if ( count($users) == 1 ) {
			return $users[0];
		} else if ( count($users) > 1 ) {
			$error = new WP_Error( 'elvez_phone_number_util_user_duplicated', __( 'Found dupulicate users have same phone number.', self::TEXT_DOMAIN ) );
			return $error;
		} else {
			return null;
		}
	}

	/**
	 * Get phone number from user meta
	 *
	 * @since	1.0.0
	 * @param	$user_id	int
	 * @return	$phone_number	string
	 */
	static public function get_phone_number( $user_id ) {
		$meta_key = self::USER_META_PHONE_NUMBER;
		return get_user_meta( $user_id, $meta_key, true);
	}

	/**
	 * Update phone number from user meta
	 *
	 * @since	1.0.1
	 * @since	4	Modify to function is static
	 * @since	5	Validate phone number before update
	 * @param	$user_id	int
	 * @param	$value			string Phone Number
	 * @return	$phone_number	string
	 */
	static public function update_phone_number( $user_id, $value, $validate=true ) {
		$sanitized_value = self::sanitized_phone_number( $value );
		if ( $validate && !self::validate_phone_number( $value, $user_id ) ) {
			return new WP_Error( 'elvez_phone_number_util_invalid_phonenumber', 'Invalid phone number.' );
		} else {
			$meta_key = self::USER_META_PHONE_NUMBER;
			return update_user_meta( $user_id, $meta_key, $value);
		}
	}

	/**
	 * Return sanitized number.
	 *
	 * @since	5
	 * @param  string	$number	Phone number string.
	 * @return string	$number	Sanitized number
	 */
	static public function sanitized_phone_number( $number ) {
		$raw_number = $number;
		$number = preg_replace( '/[^0-9]/', '', $number );
		return apply_filters( 'elvez_phone_number_util_sanitized_phone_number', $number, $raw_number );
	}

	/**
	 * Validate phone number format and unique
	 *
	 * @since	2
	 * @since	3	Modify user_id to optional.
	 * @since	4	Modify to function is static
	 * @param	string	$phone_number	string Phone Number
	 * @param	int		$user_id
	 * @return	bool	$is_valid
	 */
	static public function validate_phone_number( $phone_number, $user_id=0 ) {

		$is_phone_number = self::is_phone_number( $phone_number );
		$is_unique = self::is_unique( $phone_number, $user_id );

		$is_valid = $is_phone_number && $is_unique;
		$is_valid = apply_filters(
			'elvez_phone_number_util_validate_phone_number',
			$is_valid,
			$phone_number,
			$user_id
		);
		return $is_valid;
	}

	/**
	 * Validates a phone number using a regular expression.
	 *
	 * @since	2
	 * @since	4	Modify to function is static
	 * @since	5	Valid only 11 numbers.
	 * @param  string $number Phone number to validate.
	 * @return bool
	 */
	static public function is_phone_number( $number ) {
		$sanitized_number = self::sanitized_phone_number( $number );
		if ( 11 == strlen( $sanitized_number ) ) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Validates a phone number is uniqued across all users.
	 *
	 * @since	2
	 * @since	4	Modify to function is static
	 * @param	int	$user_id
	 * @param	string $phone_number Phone number to validate.
	 * @return	bool
	 */
	static public function is_unique( $phone_number, $user_id=0 ) {
		$exist_user = self::get_user_by_phone_number( $phone_number );
		if ( !$exist_user ) {
			return true;
		} else if ( !is_wp_error( $exist_user ) && $user->ID == $user_id ) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Create user with phone number
	 *
	 * @since	3
	 * @since	4	Modify to function is static
	 * @param	string	$phone_number
	 * @return	int		$user_id
	 */
	static public function create_user_with_phone_number( $phone_number ) {

		if ( self::validate_phone_number( $phone_number ) ) {
			$user_login = wp_slash( $phone_number );
			$user_pass = wp_generate_password();

			$userdata = array(
				'user_login' => $user_login,
				'user_pass' => $user_pass,
			);
			$user_id = wp_insert_user( $userdata );

			if ( !is_wp_error( $user_id ) ) {
				self::update_phone_number( $user_id, $phone_number, false);
			}
		} else {
			$user_id = null;
		}
		do_action( 'elvez_phone_number_util_create_user', $user_id, $phone_number );
		return $user_id;
	}

	/**
	 * Return SMS credentials
	 *
	 * @since	6
	 * @return	array
	 */
	static public function get_sms_credentials() {
		$defaults = array(
			'key' => null,
			'secret' => null,
			'regison' => 'ap-northeast-1',
		);
		$credentials = get_option( self::OPTION_SMS_CREDENTIALS, $defaults );
		return $credentials;
	}
	/**
	 * Return SMS sender id.
	 *
	 * @since	6
	 * @return	array
	 */
	static public function get_sms_sender_id() {
		return get_option( self::OPTION_SMS_SENDER_ID, '' );
	}

	/**
	 * Get amazon sns client
	 *
	 * @since	1.0.0
	 * @param	$credentials	array
	 */
	static public function get_sms_client( $credentials ) {

        $region = $credentials['region'];
        $key = $credentials['key'];
        $secret = $credentials['secret'];

        $version = '2010-03-31';
		$SnSclient = new SnsClient([
			'region' => $region,
			'version' => $version,
			'credentials' => array(
                'key' => $key,
                'secret' => $secret,
            ),
		]);
		return $SnSclient;
	}

	/**
	 * Send SMS
	 *
	 * @since	1.0.0
	 * @since	5	Validate phone number before send SMS.
     * @param   $credentials    array
     *      array(
	 *		    'key' => string,
     *		    'secret' => string,
     *          'region' => string,
	 *	    );
	 * @param	$phone_number	string
	 * @param	$message		string
     * @param   $sender_id      string
     * @return  $result         bool | WP_Error
	 */
	static public function send_sms( $credentials, $phone_number, $message, $sender_id=null ) {

		if ( !self::is_phone_number( $phone_number ) ) {
			return new WP_Error( 'elvez_phone_number_util_invalid_phone_number', __( 'Invalid Phone Number.', self::TEXT_DOMAIN ) );
		};

		$client = self::get_sms_client( $credentials );

		/**
		 * 送信の設定
		 */
		$sms_type = 'Transactional';
		$message_attrs = array(
			'AWS.SNS.SMS.SMSType' => array(
				'DataType' => 'String',
				'StringValue' => $sms_type,
			),
		);
		if ( $sender_id ) {
			$message_attrs['AWS.SNS.SMS.SenderID'] = array(
				'DataType' => 'String',
				'StringValue' => $sender_id,
			);
		}

		$country_code = '+81'; // FIXME
		$args = array(
			'MessageAttributes' => $message_attrs,
			'Message' => $message,
			'PhoneNumber' => $country_code . $phone_number,
		);
		try {
			$client->publish($args);
			$result = true;
		} catch (\Exception $e) {
			$result = new WP_Error( 'elvez_phone_number_util_send_sms', __( 'Failed to send SMS.', self::TEXT_DOMAIN ) );
		}
		return $result;

	}

	/**
	 * Register options
	 *
	 * @since	6
	 * @param	string	$option_group	A string of option group to register setting
	 */
	public static function register_options( $option_group ) {
		$defaults = array(
			'key' => null,
			'secret' => null,
			'regison' => 'ap-northeast-1',
		);
		AdminUtil::register_option_setting( $option_group, self::OPTION_SMS_CREDENTIALS, $defaults );
		AdminUtil::register_option_setting( $option_group, self::OPTION_SMS_SENDER_ID, '' );
	}
	/**
	 * Render option settings for SMS
	 *
	 * @since	6
	 */
	static public function render_sms_settings() {
		$cred_key = self::OPTION_SMS_CREDENTIALS;
		$cred = get_option( $cred_key, array() );
		?>
		<h3 class=""><?php esc_html_e( 'SMS Settings (Amazon SNS)' , self::TEXT_DOMAIN ); ?></h3>
		<table class="form-table">
			<tr valign="top">
				<th scope="row"><?php esc_html_e( 'Access Key ID', self::TEXT_DOMAIN ); ?></th>
				<td><?php AdminUtil::render_input_text_option( $cred_key, 60, ['key'] ) ?></td>
			</tr>
			<tr valign="top">
				<th scope="row"><?php esc_html_e( 'Secret Access Key', self::TEXT_DOMAIN ); ?></th>
				<td>
					<input type="password" size="60"
						name="<?php echo "${cred_key}[secret]"; ?>"
						value="<?php echo esc_attr( $cred['secret'] ); ?>" />
				</td>
			</tr>
			<tr valign="top">
				<th scope="row"><?php esc_html_e( 'Region', self::TEXT_DOMAIN ); ?></th>
				<td><?php AdminUtil::render_input_text_option( $cred_key, 60, ['region'] ) ?></td>
			</tr>
			<tr valign="top">
				<th scope="row"><?php esc_html_e( 'Sender ID', self::TEXT_DOMAIN ); ?></th>
				<td>
					<input type="text" minlength=3 maxlength=11
						name="<?php echo esc_attr( self::OPTION_SMS_SENDER_ID ); ?>"
						value="<?php echo esc_attr( get_option(self::OPTION_SMS_SENDER_ID) ); ?>" />
					<p class="description"><?php esc_html_e( 'Leave blank to use default sender ID.' , self::TEXT_DOMAIN ); ?></p>
					<p class="description"><?php esc_html_e( 'It should contains 3-11 alphanumeric characters.' , self::TEXT_DOMAIN ); ?></p>
				</td>
			</tr>
		</table>
		<?php
	}

}