<?php
namespace Elvez;

use WP_Error;
use Elvez\AdminUtilV9 as AdminUtil;


class TTSUtilV1 {
	/**
	 * 翻訳テキストドメイン
	 *
	 * @since    1
	 */
	const TEXT_DOMAIN = 'elvez_tts_util';
	/**
	 * オプションフィールド名のプレフィックス
	 * @since    1
	 */
	const OPTION_PREFIX = 'elv_tts_';

	/**
	 * AWS APIアクセスキーのオプション名
	 * @since    1
	 */
	const OPTION_AWS_ACCESS_KEY = self::OPTION_PREFIX . 'aws_access_key';
	/**
	 * AWS シークレットキーのオプション名
	 * @since    1
	 */
	const OPTION_AWS_SECRET_KEY = self::OPTION_PREFIX . 'aws_secret_key';
	/**
	 * AWSのリージョン名
	 * @since    1
	 */
	const OPTION_AWS_REGION = self::OPTION_PREFIX . 'aws_region';
	/**
	 * Amazon PollyのボイスID
	 * @since    1
	 */
	const OPTION_AMAZON_POLLY_VOICE = self::OPTION_PREFIX . 'amazon_polly_voice';
	/**
	 * Amazon Pollyのレキシコン
	 * @since    1
	 */
	const OPTION_AMAZON_POLLY_LEXICONS = self::OPTION_PREFIX . 'amazon_polly_lexicons';
	/**
	 * Amazon Pollyのブレーク秒数
	 * float	秒
	 * @since    1
	 */
	const OPTION_AMAZON_POLLY_BREAK_TIME = self::OPTION_PREFIX . 'amazon_polly_break_time';
	/**
	 * Amazon PollyのVocal Tract Length
	 * float	%
	 * @since    1
	 */
	const OPTION_AMAZON_POLLY_VTL = self::OPTION_PREFIX . 'amazon_polly_vtl';
	/**
	 * Amazon Pollyのピッチ
	 * ptich	%
	 * @since    1
	 */
	const OPTION_AMAZON_POLLY_PITCH = self::OPTION_PREFIX . 'amazon_polly_pitch';
	/**
	 * Amazon Pollyのphonation=soft設定
	 * int	0 or 1
	 * @since    1
	 */
	const OPTION_AMAZON_POLLY_PHONATION_SOFT = self::OPTION_PREFIX . 'amazon_polly_phonation_soft';

	/**
	 * The singletone instance
	 *
	 * @since    1
	 */
	protected static $_instance = null;
	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1
	 */
	public function __construct() {
		/**
		 * 翻訳ドメイン登録
		 */
		add_action( 'plugins_loaded', [$this, 'load_textdomain'] );

		// API登録
		$this->api_actions = array(
			array( 'elvez_tts', 'ELVEZ_TTS', 'handle_ajax_elvez_tts' ),
		);
		add_action( 'init', [$this, 'register_api_actions'] );
		add_action( 'wp_enqueue_scripts', [$this, 'register_ajax_scripts'] );
	}
    /**
     * Return singleton instance.
     *
     * @since   1
     * @return  TTSUtil
     */
    public static function get_instance() {
        if ( is_null( self::$_instance ) ) {
            self::$_instance = new self();
        }
        return self::$_instance;
    }
	/**
	 * Load language files.
	 *
	 * @since	1
	 */
	function load_textdomain() {
		load_plugin_textdomain(
			self::TEXT_DOMAIN,
			false,
			dirname( plugin_basename( __FILE__ ) ) . '/languages/'
		);
	}

	/**
	 * Register options
	 *
	 * @since	1
	 * @param	string	$option_group	A string of option group to register setting
	 */
	public static function register_options( $option_group ) {
		AdminUtil::register_option_setting( $option_group, self::OPTION_AWS_ACCESS_KEY, '' );
		AdminUtil::register_option_setting( $option_group, self::OPTION_AWS_SECRET_KEY, '' );
		AdminUtil::register_option_setting( $option_group, self::OPTION_AWS_REGION, 'ap-northeast-1' );

		AdminUtil::register_option_setting( $option_group, self::OPTION_AMAZON_POLLY_VOICE, 'Mizuki' );
		AdminUtil::register_option_setting( $option_group, self::OPTION_AMAZON_POLLY_LEXICONS, 'lexicons dialogues' );
		AdminUtil::register_option_setting( $option_group, self::OPTION_AMAZON_POLLY_BREAK_TIME, '0.5' );
		AdminUtil::register_option_setting( $option_group, self::OPTION_AMAZON_POLLY_VTL, '-10' );
		AdminUtil::register_option_setting( $option_group, self::OPTION_AMAZON_POLLY_PITCH, '+30' );
		AdminUtil::register_option_setting( $option_group, self::OPTION_AMAZON_POLLY_PHONATION_SOFT, '1' );

	}
	/**
	 * Render option form to firebase API
	 *
	 * @since	1
	 */
	public static function render_option_settings() {
		?>
		<h3 class=""><?php esc_html_e( 'Elvez TTS Settings' , self::TEXT_DOMAIN ); ?></h3>
		<p class="description"><?php esc_html_e( 'This setting may be overridden by other plugins.', self::TEXT_DOMAIN ); ?></p>
		<?php
		self::render_aws_settings();
		self::render_tts_settings();
	}
	/**
	 * Render AWS option form
	 *
	 * @since	1
	 */
	public static function render_aws_settings() {
		?>
		<h4 class=""><?php esc_html_e( 'AWS Settings' , self::TEXT_DOMAIN ); ?></h4>
		<table class="form-table">
			<tr valign="top">
				<th scope="row"><?php esc_html_e( 'AWS Access Key', self::TEXT_DOMAIN ); ?></th>
				<td><?php AdminUtil::render_input_text_option( self::OPTION_AWS_ACCESS_KEY ) ?></td>
			</tr>
			<tr valign="top">
				<th scope="row"><?php esc_html_e( 'AWS Secret Key', self::TEXT_DOMAIN ); ?></th>
				<td>
					<input type="password" size="60"
						name="<?php echo self::OPTION_AWS_SECRET_KEY; ?>"
						value="<?php echo esc_attr( get_option(self::OPTION_AWS_SECRET_KEY) ); ?>" />
				</td>
			</tr>
			<tr valign="top">
				<th scope="row"><?php esc_html_e( 'AWS Region', self::TEXT_DOMAIN ); ?></th>
				<td><?php AdminUtil::render_input_text_option( self::OPTION_AWS_REGION ) ?></td>
			</tr>
		</table>
		<?php
	}
	/**
	 * Render Polly option form
	 *
	 * @since	1
	 */
	public static function render_tts_settings() {
		?>
		<h4 class=""><?php esc_html_e( 'TTS Settings' , self::TEXT_DOMAIN ); ?></h4>
		<table class="form-table">
			<tr valign="top">
				<th scope="row"><?php esc_html_e( 'AWS Access Key', self::TEXT_DOMAIN ); ?></th>
				<td><?php AdminUtil::render_input_text_option( self::OPTION_AMAZON_POLLY_VOICE ) ?></td>
			</tr>
			<tr valign="top">
				<th scope="row"><?php esc_html_e( 'AWS Access Key', self::TEXT_DOMAIN ); ?></th>
				<td><?php AdminUtil::render_input_text_option( self::OPTION_AMAZON_POLLY_LEXICONS ) ?></td>
			</tr>
			<tr valign="top">
				<th scope="row"><?php esc_html_e( 'AWS Access Key', self::TEXT_DOMAIN ); ?></th>
				<td><?php AdminUtil::render_input_text_option( self::OPTION_AMAZON_POLLY_BREAK_TIME ) ?></td>
			</tr>
			<tr valign="top">
				<th scope="row"><?php esc_html_e( 'AWS Access Key', self::TEXT_DOMAIN ); ?></th>
				<td><?php AdminUtil::render_input_text_option( self::OPTION_AMAZON_POLLY_VTL ) ?></td>
			</tr>
			<tr valign="top">
				<th scope="row"><?php esc_html_e( 'AWS Access Key', self::TEXT_DOMAIN ); ?></th>
				<td><?php AdminUtil::render_input_text_option( self::OPTION_AMAZON_POLLY_VOICE ) ?></td>
			</tr>
			<tr valign="top">
				<th scope="row"><?php esc_html_e( 'AWS Access Key', self::TEXT_DOMAIN ); ?></th>
				<td><?php AdminUtil::render_input_text_option( self::OPTION_AMAZON_POLLY_PITCH ) ?></td>
			</tr>
			<tr valign="top">
				<th scope="row"><?php esc_html_e( 'AWS Access Key', self::TEXT_DOMAIN ); ?></th>
				<td><?php AdminUtil::render_input_text_option( self::OPTION_AMAZON_POLLY_PHONATION_SOFT ) ?></td>
			</tr>
		</table>
		<?php
	}


	/**
	 * Register actions for APIs.
	 *
	 * @since	1
	 */
	function register_api_actions() {
		foreach( $this->api_actions as $args ) {
			$action = $args[0];
			$callback = $args[2];
			add_action( 'wp_ajax_' . $action, [$this, $callback] );
		}
	}
	/**
	 * Register ajax API scripts.
	 *
	 * @since     1
	 */
	function register_ajax_scripts() {
		foreach( $this->api_actions as $args ) {
			$action = $args[0];
			$var_name = $args[1];
			wp_localize_script( 'jquery', $var_name, [
				'api'    => admin_url( 'admin-ajax.php' ),
				'action' => $action,
				'nonce'  => wp_create_nonce( $action )
			]);
		}
	}
	/**
	 * Validate API Request.
	 *
	 * @since 1
	 * @param $action	string
	 */
	function validate_api_action( $action ) {
		if( !check_ajax_referer( $action, 'nonce', false ) ) {
			status_header( '403' );
			$data['msg'] = 'Forbidden';
			exit();
		}
	}
	/**
	 *  TTS API.
	 *
	 * @since     1
	 * @return    string	JSON string
	 */
	function handle_ajax_elvez_tts() {
		$action = 'elvez_tts';
		$this->validate_api_action( $action );

		$text = isset( $_POST['text'] ) ? esc_attr( $_POST['text'] ) : null;
		if ( !$text ) {
			status_header( '400' );
			exit();
		}

		// Get client
		$region = get_option(self::OPTION_AWS_REGION);
		$api_key = get_option(self::OPTION_AWS_ACCESS_KEY);
		$api_secret = get_option(self::OPTION_AWS_SECRET_KEY);

		$aws_sdk_config = array(
			'region'      => $region,
			'version'     => 'latest',
			'credentials' => array(
					'key'    => $api_key,
					'secret' => $api_secret ,
			),
		);
		$sdk = new Aws\Sdk( $aws_sdk_config );
		$client = $sdk->createPolly();

		// Get audio stream
		$voice_id = get_option(self::OPTION_AMAZON_POLLY_VOICE);
		$lexicons = get_option(self::OPTION_AMAZON_POLLY_LEXICONS);
		$break_time = get_option(self::OPTION_AMAZON_POLLY_BREAK_TIME);
		$vtl = get_option(self::OPTION_AMAZON_POLLY_VTL);
		$pitch = get_option(self::OPTION_AMAZON_POLLY_PITCH);
		$soft = get_option(self::OPTION_AMAZON_POLLY_PHONATION_SOFT);

		$lexicons_array = explode( ' ', trim( $lexicons ) );
		if ($soft) {
			$phonation = 'phonation="soft" ';
		} else {
			$phonation = '';
		}

		$break_tag = '<break time="' . $break_time . 's"/>';
		$text = str_replace(array(" ", "　"), $break_tag, $text);
		$ssml_text = '<speak><amazon:effect ' . $phonation . 'vocal-tract-length="' . $vtl . '%"><prosody pitch="' . $pitch . '%">' . $text . '</prosody></amazon:effect></speak>';

		$result = $client->synthesizeSpeech(
			array(
				'OutputFormat' => 'mp3',
				'Text'         => $ssml_text,
				'TextType'     => 'ssml',
				'VoiceId'      => $voice_id,
				'LexiconNames' => $lexicons_array,
			)
		);

		// Convert audio stream to base64
		if ( isset($result['AudioStream']) ) {
			$contents = $result['AudioStream']->getContents();

			$data['speech'] = base64_encode($contents);
		} else {
			$data['msg'] = 'Request Error';
		}


		header( 'Content-Type: application/json; charset=UTF-8' );
		echo json_encode($data);

		exit();
	}


}